package net.willware.Orbits;

import java.awt.Graphics;

public interface SimObject {
	public void timeStep1();
	public void timeStep2();
	public void write();
	public void interact(SimObject otherguy);
	public void draw(Graphics g, Grapher gg);
	public void growBoundingBox(Grapher gg);
}
